// Fetch Text URL 1.6.4
// Jeff Chien - June 30, 2009

const ftu_preferences = Components.classes["@mozilla.org/preferences-service;1"].
							getService(Components.interfaces.nsIPrefService);
const ftu_branch = ftu_preferences.getBranch('ftu.');
const ftu_protocols = ["http","https","ftp"];
const ftu_whiteSpace = " \t\n";
const ftu_alphanum = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789()";
// valid urls can contain the chars "$-_.+!*'()," and alphanumeric characters
var FTU = {
	init: function()
	{
		var menu = document.getElementById("contentAreaContextMenu");
		menu.addEventListener("popupshowing", FTU.onPopupShowing, false);

		var newtab = null;
		try {
			newtab = ftu_branch.getBoolPref('newTab');
		} catch(e) { ftu_branch.setBoolPref('newTab', true); }

		var curtab = null;
		try {
			curtab = ftu_branch.getBoolPref('currentTab');
		} catch(e) { ftu_branch.setBoolPref('currentTab', true); }

		var newwin = null;
		try {
			newwin = ftu_branch.getBoolPref('newWindow');
		} catch(e) { ftu_branch.setBoolPref('newWindow', false); }

		var dispurl = null;
		try {
			dispurl = ftu_branch.getCharPref('displayURL');
		} catch(e) { ftu_branch.setCharPref('displayURL', 'showurl'); }

		var icons = null;
		try {
			icons = ftu_branch.getBoolPref('showIcons');
		} catch(e) { ftu_branch.setBoolPref('showIcons', true); }
	},

	// searchSelected() taken from 1.8.0 branch ContextMenu.js
	searchSelected: function() {
		var focusedWindow = document.commandDispatcher.focusedWindow;
		var searchStr = focusedWindow.getSelection();
		searchStr = searchStr.toString();
		searchStr = searchStr.replace( /^\s+/, "" );
		searchStr = searchStr.replace(/(\n|\r|\t)+/g, " ");
		searchStr = searchStr.replace(/\s+$/,"");
		return searchStr;
	},

	fetch: function(aWinPref)
	{
		var str = this.reform(new String(this.searchSelected()));

		switch(aWinPref) {
			case 0:	// Open in current window.
					gBrowser.loadURI(str);
					break;

			case 1:	// Open in new tab.
					var newTab = window.getBrowser().addTab(str);
					if(!ftu_preferences.getBoolPref("browser.tabs.loadInBackground")) {
						window.getBrowser().selectedTab = newTab;
					}
					break;

			case 2:	// Open in new window.
					window.open(str, null);
					break;
		}
	},

	reform: function(aStrURL)
	{
		var str = aStrURL;
		var protocolFound = false;

		// Java-like trim() operation, slightly modified.
		// Assumes valid URIs start and end with alphanumeric chars.
		while(str.length > 0 && (ftu_whiteSpace.indexOf(str.charAt(0)) >= 0
				|| ftu_alphanum.indexOf(str.charAt(0)) < 0) ) {
			str = str.substring(1);
		}
		while(str.length > 0 && str.charAt(str.length - 1) != '/' &&
				(ftu_whiteSpace.indexOf(str.charAt(str.length - 1)) >= 0
				|| ftu_alphanum.indexOf(str.charAt(str.length - 1)) < 0) ) {
			str = str.substring(0, str.length - 1);
		}

		var temp = "";
		for(var i=0;i<str.length;i++) {
			if(ftu_whiteSpace.indexOf(str.charAt(i)) < 0) {
				temp+=str.charAt(i);
			}
		}
		str = temp;

		for(var i=0;i<ftu_protocols.length;i++) {
			if(str.indexOf(ftu_protocols[i] + "://") == 0)
				protocolFound = true;
		}

		if(!protocolFound) {

			if(str.indexOf("://") >= 0) // protocol is malformed
				str = str.substring(str.indexOf("://") + 3);

			str = "http://" + str;		// assume http protocol is desired
		}

		return str;
	},

	onPopupShowing: function()	// can't seem to reference an instanced FTU object from here
	{
		var selected;

		if(ftu_branch.getCharPref('displayURL') == 'showurl') {
			selected = FTU.reform(new String(FTU.searchSelected()));

			if(selected.length > 22) {
				// Truncate long URLs for aesthetics
				selected = selected.substring(0,22) + "...";
			}

			selected = "\"" + selected + "\"";
		}
		else {
			selected = "text URL";
		}
        gContextMenu.isTextSelected; // (hopefully) temporary work around for compatibility with menu editor
		// I should probably put the various arguments in arrays to make it prettier.
		FTU.display("context-fetchnewtab", 'newTab', selected, "new tab","newtab.png");
		FTU.display("context-fetchcurtab",'currentTab', selected, "current tab","curtab.png");
		FTU.display("context-fetchnewwin",'newWindow', selected, "new window","newwin.png");
		FTU.displaySep("context-fetchseparator");
	},

	display: function(aStrID, aStrPrefID, aStrText, aStrLoc, aStrIcon)
	{
		var menuitem = document.getElementById(aStrID);

		if(menuitem) {
			menuitem.label = "Fetch " + aStrText + " in " + aStrLoc;
			if(!ftu_branch.getBoolPref(aStrPrefID))
				menuitem.hidden = true;
			else
				menuitem.hidden = !gContextMenu.isTextSelected;
		}

		if(ftu_branch.getBoolPref('showIcons')) {
			document.getElementById(aStrID).style.listStyleImage = "url('chrome://ftu/skin/" + aStrIcon + "')";
		}
		else {
			document.getElementById(aStrID).style.listStyleImage = 'none';
		}
	},

	displaySep: function(aStrID)
	{
		var menuitem = document.getElementById(aStrID);
		if(menuitem) {
			if(!ftu_branch.getBoolPref('newTab') && !ftu_branch.getBoolPref('currentTab')
				&& !ftu_branch.getBoolPref('newWindow')) {
				menuitem.hidden = true;
			}
			else {
				menuitem.hidden = !gContextMenu.isTextSelected;
			}
		}
	}
};

window.addEventListener("load", FTU.init, false);
